/*
 * ----------------------------------------------------------------
 * ircproxy - Connection Logging Header
 * ----------------------------------------------------------------
 * Copyright (C) 1997-2009 Jonas Kvinge
 *
 * This file is part of ircproxy.
 *
 * ircproxy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ircproxy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ircproxy.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional permission under GNU GPL version 3 section 7
 *
 * If you modify ircproxy, or any covered work, by linking or
 * combining it with openssl (or a modified version of that library),
 * containing parts covered by the terms of the OpenSSL License and the
 * SSLeay License, the licensors of ircproxy grant you additional
 * permission to convey the resulting work.
 *
 * $Id: conn_log.h 54 2009-03-18 18:23:29Z jonasio $
 *
 */

#ifdef CONN_LOG_H
#warning "conn_log.h already included."
#else
#define CONN_LOG_H

#define CONN_LOG_TYPE_RAW			1
#define CONN_LOG_TYPE_PRIV			2
#define CONN_LOG_TYPE_CHAN			3

#define ConnLogError(...) \
		do { \
			time_t Duration = (NOW - ConnS->LogError); \
			if (Duration > 3600) { \
				sysprint(BITMASK_ERROR, __VA_ARGS__); \
				Conn_SetLogError(ConnS); \
				ConnS->LogError = NOW; \
			} \
		} \
		while (0) \

/* STRUCTURES - JONAS (20.01.2008) */

struct ConnLog_Struct {

  char *Name;
  char *LogFile;
  char *LogFileFullPath;
  unsigned short int LogType;

  struct ConnLog_Struct *Next;
  struct ConnLog_Struct *Prev;

};

struct Variable_Struct {

  const char *Name;
  const char *Data;

};

struct VariableData_Struct {

  const struct Conn_Struct *ConnS;
  const struct Chan_Struct *ChanS;
  const char *Chan;
  const char *Nick;
  const char *User;
  const char *Host;
  const char *NUH;
  const char *Command;
  const char *Param;
  const char *Message;

};

#ifdef CONN_LOG_C
  struct VariableData_Struct VariableDataS = { 0 };
#else
  extern struct VariableData_Struct VariableDataS;
#endif

/* FUNCTION PROTOTYPES - JONAS (24.06.2000) */

struct ConnLog_Struct *conn_addlog(struct Conn_Struct *ConnS, unsigned short int LogType, const char *const NamePT, const char *const LogFilePT, const char *const LogFileFullPathPT);
void conn_remlog(struct Conn_Struct *ConnS, struct ConnLog_Struct *ConnLog);
struct ConnLog_Struct *conn_getlog(struct Conn_Struct *ConnS, const char *const LogfilePT);
void conn_destroylogs(struct Conn_Struct *ConnS);

unsigned short int conn_log(struct Conn_Struct *ConnS, struct Chan_Struct *ChanS, const char *const ChanPT, const char *const NickPT, const char *const UserPT, const char *const HostPT, const char *const NUHPT, const char *const CommandPT, const char *const ParamPT, const char *const MessagePT);
void conn_rawlog(struct Conn_Struct *ConnS, const char *const LinePT, ...);
char *conn_log_variable(const char *OldMessagePT, char *NewMessagePT);

#endif
