/*
 * ----------------------------------------------------------------
 * ircproxy - Connection Parser Header
 * ----------------------------------------------------------------
 * Copyright (C) 1997-2009 Jonas Kvinge
 *
 * This file is part of ircproxy.
 *
 * ircproxy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ircproxy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ircproxy.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional permission under GNU GPL version 3 section 7
 *
 * If you modify ircproxy, or any covered work, by linking or
 * combining it with openssl (or a modified version of that library),
 * containing parts covered by the terms of the OpenSSL License and the
 * SSLeay License, the licensors of ircproxy grant you additional
 * permission to convey the resulting work.
 *
 * $Id: conn_parser.h 54 2009-03-18 18:23:29Z jonasio $
 *
 */

#ifdef CONN_PARSER_H
#warning "conn_parser.h already included."
#else
#define CONN_PARSER_H

/* MACROS - JONAS (24.06.2000) */

#define CONN_PARSE_STRREALLOC(P, S) \
  strrealloc((P), (S)); \
  if (aerrno != AESUCCESS) { \
    assert(Conn_IsSocket(ConnS)); \
    if (Conn_IsWelcome(ConnS)) { conn_quit(ConnS, "String reallocation failure in function %s() (%s:%d): [%d] %s", __FUNCTION__, __FILE__, __LINE__, aerrno, straerror(aerrno)); } \
    else { conn_disconnect(ConnS, "Connection %s server %s: String reallocation failure in function %s() (%s:%d): [%d] %s", ConnS->Name, ConnS->ServerHostName, __FUNCTION__, __FILE__, __LINE__, aerrno, straerror(aerrno)); } \
    return(FALSE); \
  }

#define CONN_PARSE_NUMERIC_ERROR(X) \
	conn_quit(ConnS, \
	"Internal failure calling function %s() from function %s() (%s:%d) connection: %s after receiving numeric %d from server %s(%s):%ld: [%d] %s" \
	, (X), __FUNCTION__, __FILE__, __LINE__, ConnS->Name, Numeric, ConnS->ServerHostName, ConnS->ServerHostIPS, ConnS->ServerPortH, aerrno, straerror(aerrno)); \
	return(FALSE)

#define CONN_PARSE_NUMERIC_CHECKPARAMS(X) \
	if (Params < X) { \
		if (Conn_IsWelcome(ConnS)) { conn_quit(ConnS, "Too few parameters for numeric %d received from server %s(%s):%ld (function %s (%s:%d))", Numeric, ConnS->ServerHostName, ConnS->ServerHostIPS, ConnS->ServerPortH, __FUNCTION__, __FILE__, __LINE__); } \
		else { conn_disconnect(ConnS, "Connection %s: Too few parameters for numeric %d received from server %s(%s):%ld (function %s (%s:%d))", ConnS->Name, Numeric, ConnS->ServerHostName, ConnS->ServerHostIPS, ConnS->ServerPortH, __FUNCTION__, __FILE__, __LINE__); } \
		return(FALSE); \
	}


#define CONN_PARSE_EVENT_ERROR(X) \
	conn_quit(ConnS, \
	"Internal failure calling function %s() from function %s() (%s:%d) connection: %s after receiving event %s from server %s(%s):%ld: [%d] %s", \
	(X), __FUNCTION__, __FILE__, __LINE__, ConnS->Name, CommandPT, ConnS->ServerHostName, ConnS->ServerHostIPS, ConnS->ServerPortH, aerrno, straerror(aerrno)); \
	return(FALSE)

#define CONN_PARSE_EVENT_CHECKPARAMS(X) \
	if (Params < X) { \
		if (Conn_IsWelcome(ConnS)) { conn_quit(ConnS, "Too few parameters for event %s received from server %s(%s):%ld (function %s (%s:%d))", CommandPT, ConnS->ServerHostName, ConnS->ServerHostIPS, ConnS->ServerPortH, __FUNCTION__, __FILE__, __LINE__); } \
		else { conn_disconnect(ConnS, "Connection %s: Too few parameters for event %s received from server %s(%s):%ld (function %s (%s:%d))", ConnS->Name, CommandPT, ConnS->ServerHostName, ConnS->ServerHostIPS, ConnS->ServerPortH, __FUNCTION__, __FILE__, __LINE__); } \
		return(FALSE); \
	}


/* TYPEDEFS - JONAS (24.06.2000) */

#define CONN_PARSE_NUMERIC(X)			unsigned short int X( \
						struct Conn_Struct *ConnS, \
						const char *const LinePT, \
						const char *const PrefixPT, \
						const unsigned short int Numeric, \
						char **ParamsPT, \
						const unsigned short int Params, \
						const char *ParamsStrPT \
						)

#define CONN_PARSE_EVENT(X)			unsigned short int X( \
						struct Conn_Struct *ConnS, \
						const char *const LinePT, \
						const char *const NickPT, \
						const char *const UserPT, \
						const char *const HostPT, \
						const char *const NUHPT, \
						const char *const CommandPT, \
						char **ParamsPT, \
						const unsigned short int Params, \
						const char *ParamsStrPT \
						)

/* FUNCTION PROTOTYPES - JONAS (18.07.2001) */

void conn_parser(struct Conn_Struct *ConnS);
void conn_parse_message(struct Conn_Struct *ConnS, char *MessagePT);
CONN_PARSE_NUMERIC(conn_parse_numeric);
CONN_PARSE_NUMERIC(conn_parse_numeric_welcome);
CONN_PARSE_NUMERIC(conn_parse_numeric_isupport);
CONN_PARSE_NUMERIC(conn_parse_numeric_motd);
CONN_PARSE_NUMERIC(conn_parse_numeric_nowaway);
CONN_PARSE_NUMERIC(conn_parse_numeric_unaway);
CONN_PARSE_NUMERIC(conn_parse_numeric_nextnick);
CONN_PARSE_NUMERIC(conn_parse_numeric_ison);
CONN_PARSE_NUMERIC(conn_parse_numeric_namereply);
CONN_PARSE_NUMERIC(conn_parse_numeric_endofnames);
CONN_PARSE_NUMERIC(conn_parse_numeric_whoreply);
CONN_PARSE_NUMERIC(conn_parse_numeric_endofwho);
CONN_PARSE_NUMERIC(conn_parse_numeric_topic);
unsigned short int conn_parse_event(struct Conn_Struct *ConnS, const char *const LinePT, const char *const PrefixPT, const char *const CommandPT, char **ParamsPT, const unsigned short int Params, const char *ParamsStrPT);
CONN_PARSE_EVENT(conn_parse_event_ping);
CONN_PARSE_EVENT(conn_parse_event_pong);
CONN_PARSE_EVENT(conn_parse_event_error);
CONN_PARSE_EVENT(conn_parse_event_nick);
CONN_PARSE_EVENT(conn_parse_event_join);
CONN_PARSE_EVENT(conn_parse_event_part);
CONN_PARSE_EVENT(conn_parse_event_quit);
CONN_PARSE_EVENT(conn_parse_event_kick);
CONN_PARSE_EVENT(conn_parse_event_mode);
CONN_PARSE_EVENT(conn_parse_event_topic);
CONN_PARSE_EVENT(conn_parse_event_privmsg);
CONN_PARSE_EVENT(conn_parse_event_notice);

#endif
