/*
 * ----------------------------------------------------------------
 * IRC Defines
 * ----------------------------------------------------------------
 * Copyright (C) 1997-2009 Jonas Kvinge
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $Id: irc_defines.h 54 2009-03-18 18:23:29Z jonasio $
 *
 *
 */

#ifdef IRC_DEFINES_H
#warning "irc_defines.h already included."
#else
#define IRC_DEFINES_H


/* Network bitmasks */

#define IRC_NETWORK_ZERO						0
#define IRC_NETWORK_GENERIC						1
#define IRC_NETWORK_IRCNET						2
#define IRC_NETWORK_EFNET						4
#define IRC_NETWORK_UNDERNET						8
#define IRC_NETWORK_DALNET						16
#define IRC_NETWORK_FREENODE						32
#define IRC_NETWORK_ALL							(IRC_NETWORK_GENERIC|IRC_NETWORK_IRCNET|IRC_NETWORK_EFNET|IRC_NETWORK_UNDERNET|IRC_NETWORK_DALNET|IRC_NETWORK_FREENODE)


/* Unknown NICK/USER/HOST strings */

#define IRC_UNKNOWNNICK							"."
#define IRC_UNKNOWNUSER							"."
#define IRC_UNKNOWNHOST							"."
#define IRC_UNKNOWNNUH							IRCUNKNOWNNICK "!" IRCUNKNOWNUSER "@" IRCUNKNOWNHOST


/* Characters */

#define IRC_NUHCHARS							"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-._!@`[]{}|\\^?*~"
#define IRC_NICKCHARS							"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_`[]{}|\\^?*"
#define IRC_USERCHARS							"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_.?*~"
#define IRC_HOSTCHARS							"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-.?*"
#define IRC_PASSCHARS							"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-.?*"
#define IRC_SERVERCHARS							IRC_HOSTCHARS

#define IRC_EVENTS							"abcdiew"
#define IRC_CHANEVENTS							"jpqknmtabcdxz"

#define IRC_USERMODES							"+-iws"
#define IRC_CHANMODES							"+-ilpskmnt"


/* SendQ */

#define IRC_SENDMAXLINES						100
#define IRC_SENDLINETIME						1

#define IRC_SENDMAXBUFFER						1024
#define IRC_SENDBUFFERTIME						120


/* Settings */

#define IRC_MAXPARAMS							15
#define IRC_MAXMODES							3
#define IRC_MAXCHANS							10
#define IRC_CHANMAXBANS							30


/* Lengths */

#define IRC_SERVERLEN_FIXED						IRC_HOSTLEN_FIXED
#define IRC_SERVERLEN_MAX						IRC_HOSTLEN_MAX
#define IRC_SERVERINFOLEN_FIXED						200

#define IRC_PASSLEN							50

#define IRC_NICKLEN_DEFAULT						9
#define IRC_NICKLEN_FIXED						50

#define IRC_USERLEN_FIXED						30

#define IRC_HOSTLEN_FIXED						200
#define IRC_HOSTLEN_MAX							63

#define IRC_NAMELEN_FIXED						50

#define IRC_USERHOSTLEN_FIXED						(IRC_NICKLEN_FIXED + IRC_USERLEN_FIXED + 1)

#define IRC_NUHLEN_FIXED						(IRC_NICKLEN_FIXED + IRC_USERLEN_FIXED + IRC_HOSTLEN_FIXED + 2)

#define IRC_MSGLEN							510
#define IRC_MSGCRLFLEN							(IRC_MSGLEN+2)

#define IRC_PRIVMSGLEN							490
#define IRC_NOTICELEN							490
#define IRC_CTCPLEN							490
#define IRC_ACTIONLEN							490
#define IRC_QUITMSGLEN							200
#define IRC_KILLMSGLEN							200
#define IRC_WALLOPSMSGLEN						200
#define IRC_AWAYMSGLEN							200

#define IRC_MODECHARLEN							2
#define IRC_MODEPARAMLEN						50

#define IRC_MODECHARSLEN						10
#define IRC_MODEPARAMSLEN						510

#define IRC_MODELINELEN							(IRC_MODECHARSLEN + IRC_MODEPARAMSLEN)
#define IRC_MODESLEN							(IRCCHANMODELEN * IRCCHANMAXMODES)

#define IRC_CHANLEN							50
#define IRC_CHANKEYLEN							23
#define IRC_CHANKICKMSGLEN						200
#define IRC_CHANPARTMSGLEN						200
#define IRC_CHANTOPICLEN						160

/* IRC Mask defines */

#define IRC_MINNICKMASK							1
#define IRC_MAXNICKMASK							6
#define IRC_MINUSERMASK							1
#define IRC_MAXUSERMASK							6
#define IRC_MINHOSTMASK							1
#define IRC_MAXHOSTMASK							6

/* Structures */

struct IRC_ISupport_Struct {

  char *Network;
  unsigned long int Networks;
  unsigned short int MaxModes;
  unsigned short int MaxChans;
  unsigned short int MaxBans;
  unsigned short int NickLen;
  unsigned short int AwayMsgLen;
  unsigned short int TopicLen;
  unsigned short int KickMsgLen;
  char *ChanTypes;
  char *Prefix;
  char *ChanModes;

};

struct IRC_ISupport_ChanMode_Struct {

  char Mode;
  unsigned short int Param;

};

struct IRC_ISupport_ChanUserMode_Struct {

  char Mode;
  char Prefix;

};


#endif
