/*
 * ----------------------------------------------------------------
 * libmemcalls - Memory Allocation Library Header
 * ----------------------------------------------------------------
 * Copyright (C) 1997-2009 Jonas Kvinge
 *
 * This file is part of libmemcalls.
 *
 * libmemcalls is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * libmemcalls is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $Id: memcalls.h 54 2009-03-18 18:23:29Z jonasio $
 *
 */

#ifdef MEMCALLS_H
#warning "memcalls.h already included."
#else
#define MEMCALLS_H

#ifndef MEMCALLS_C
  #define MALLOC_REDEFINE
  #undef malloc
  #undef realloc
  #undef calloc
  #undef free
  #undef strdup
  #define malloc(s) memcalls_malloc(__FILE__, __LINE__, __FUNCTION__, (s))
  #define realloc(p, s) memcalls_realloc(__FILE__, __LINE__, __FUNCTION__, (p), (s))
  #define calloc(n, s) memcals_calloc(__FILE__, __LINE__, __FUNCTION__, (n), (s))
  #define free(p) memcalls_free(__FILE__, __LINE__, __FUNCTION__, (p))
  #define strdup(p) memcalls_strdup(__FILE__, __LINE__, __FUNCTION__, (p))
#endif

/* FUNCTION PROTOTYPES - JONAS (21.09.2001) */

void *memcalls_malloc(const char *const FilePT, const unsigned short int Line, const char *const FunctionPT, const size_t MemSize);
void *memcalls_realloc(const char *const FilePT, const unsigned short int Line, const char *const FunctionPT, void *const OldPTR, const size_t MemSize);
void *memcalls_calloc(const char *const FilePT, const unsigned short int Line, const char *const FunctionPT, const size_t Elements, const size_t Size);
signed short int memcalls_free(const char *const FilePT, const unsigned short int Line, const char *const FunctionPT, void *const MemPTR);
char *memcalls_strdup(const char *const FilePT, const unsigned short int Line, const char *const FunctionPT, const char *StringPT);

#endif
